#include <windows.h>
#include <QtGui>
#include "vi.h"
#include "ViEditView.h"
#include "ViEngine.h"
#include <QElapsedTimer>
#include <QDebug>

ViEditView::ViEditView(QWidget *parent)
	: m_mode(CMD), QPlainTextEdit(parent)
{
	m_timer = new QElapsedTimer;
	m_timer->start();
	m_drawText = true;
	setTabStopWidth(fontMetrics().width(' ') * 4);		//	tab 4
#if !USE_EVENT_FILTER
	m_viEngine = 0;
#endif
	connect(this, SIGNAL(cursorPositionChanged()), this, SLOT(onCursorPositionChanged()));
	m_lineNumberArea = new QWidget(this);
	m_lineNumberArea->installEventFilter(this);
	//onLineNumberAreaWidthChanged();
	m_lineNumberWidth = fontMetrics().width('8') * 6;
	m_lineNumberAreaWidth = fontMetrics().width('8') * 8;
	setViewportMargins(m_lineNumberAreaWidth, 0, 0, 0);

	setCursorWidth(0);				//	QPlainTextEdit pӂĂJ[\͔\
	onCursorPositionChanged();
}

ViEditView::~ViEditView()
{
}
void ViEditView::setDrawText(bool b)
{
	m_drawText = b;
	repaint();
}
void ViEditView::focusInEvent ( QFocusEvent * event )
{
	if( mode() == CMDLINE )
		emit focusInned();
	else
		QPlainTextEdit::focusInEvent ( event );
}
#if 0
void ViEditView::mousePressEvent ( QKeyEvent * event )
{
}
#endif
#if !USE_EVENT_FILTER
void ViEditView::keyPressEvent ( QKeyEvent * event )
{
	if( m_viEngine != 0 && m_viEngine->processKeyPressEvent(event) )
		return;
	QPlainTextEdit::keyPressEvent(event);
}
#endif
void ViEditView::setMode(Mode mode)
{
	m_tickCount = m_timer->elapsed();
	if( mode != m_mode ) {
		m_mode = mode;
		emit modeChanged(mode);
		onCursorPositionChanged();
	}
}
void ViEditView::onCursorPositionChanged()
{
	//if( mode() == CMD ) {
		QTextCursor cur = textCursor();
		cur.movePosition(QTextCursor::Right, QTextCursor::KeepAnchor);
		QString text = cur.selectedText();
		QChar ch = text.isEmpty() ? QChar(' ') : text[0];
		int wd = fontMetrics().width(ch);
		if( !wd )
			wd = fontMetrics().width(QChar(' '));
		//setCursorWidth(wd);
		m_cursorWidth = wd;
	//} else {
		//setCursorWidth(1);
		//m_cursorWidth = 1;
	//}
}
void ViEditView::moveCursor(QTextCursor::MoveOperation mv, int n)
{
	m_tickCount = m_timer->elapsed();
	QTextCursor cur = textCursor();
	cur.movePosition(mv, QTextCursor::MoveAnchor, n);
	setTextCursor(cur);
}
#if 0
int ViEditView::lineNumberAreaWidth() const
{
	return 32;
}
void ViEditView::onLineNumberAreaWidthChanged()
{
	setViewportMargins(lineNumberAreaWidth(), 0, 0, 0);
}
void ViEditView::lineNumberAreaPaintEvent(QPaintEvent * event)
{
}
#endif
void ViEditView::resizeEvent(QResizeEvent *event)
{
	QPlainTextEdit::resizeEvent(event);
	//QRect r = contentsRect();
	QRect r = rect();
	m_lineNumberArea->setGeometry(QRect(r.left(), r.top(), m_lineNumberAreaWidth, r.height()));
}
bool ViEditView::eventFilter(QObject *obj, QEvent *event)
{
	if( obj == m_lineNumberArea && event->type() == QEvent::Paint ) {
		drawLineNumbers();
		return true;
	}
	return false;
}

void ViEditView::drawLineNumbers()
{
	//qDebug() << "drawLineNumbers()";
	QPainter painter(m_lineNumberArea);
	painter.setPen(Qt::black);
	QRect r = m_lineNumberArea->rect();
	painter.fillRect(r, Qt::lightGray);
#if 0
	painter.drawLine(0, 0, 10, 100);
	painter.drawText(10, 10, "hello");
#endif
#if 1
	const int ht = fontMetrics().height();
	QTextBlock block = firstVisibleBlock();
    int lineNumber = block.blockNumber() + 1;
	while( block.isValid() ) {
	    const int y = (int) blockBoundingGeometry(block).translated(contentOffset()).top();
	    if( y >= r.bottom() ) break;
		QString number = QString::number(lineNumber);
		painter.drawText(0, y, m_lineNumberWidth, ht, Qt::AlignRight, number);
		++lineNumber;
		block = block.next();
	}
#endif
}

void ViEditView::paintEvent(QPaintEvent * event)
{
#if 0
	qDebug() << "event->rect() = " << event->rect();
	static int cnt = 0;
	static int color = Qt::darkGray;
	QPainter painter(this);
	painter.fillRect(event->rect(), color);
	if( ++color == Qt::transparent ) color = Qt::darkGray;
#endif
	//qDebug() << "ViEditView::paintEvent() " << ++cnt;
#if DRAW_CURSOR

	const int blinkPeriod = 1200;	//	_ŎAPʁFms
	qint64 tc = m_timer->elapsed() - m_tickCount;
	if( tc % blinkPeriod < blinkPeriod / 2 )
		drawCursor();
	//const bool ro = isReadOnly();
	//setReadOnly(true);		//	J[\`֎~ɂ邽
	if( m_drawText )
		QPlainTextEdit::paintEvent(event);
	//setReadOnly(ro);
#else
	if( m_drawText )
		QPlainTextEdit::paintEvent(event);
#endif
	//drawCursor();
	m_lineNumberArea->repaint(m_lineNumberArea->rect());
	//drawLineNumbers();
#if 0
	QPainter painter(viewport());
	//QPainter painter(this);
#if 0
	painter.fillRect(event->rect(), Qt::lightGray);
	QRect viewportRect = viewport()->rect();
	QRect r = rect();
	painter.setPen(Qt::black);
	painter.drawLine(r.x(), r.y(), r.x() + r.width(), r.y() + r.height());
	painter.drawText(10, 10, "hello");
#endif
#if 1
	QTextBlock block = firstVisibleBlock();
	//int top = (int) blockBoundingGeometry(block).translated(contentOffset()).top();
	//int bottom = top + (int) blockBoundingRect(block).height();
	int x = 0;
	int y = 0;
	while( block.isValid() ) {
		y = (int) blockBoundingGeometry(block).translated(contentOffset()).top();
		const int ht = blockBoundingRect(block).height();
		qDebug() << "y = " << y << ", ht = " << ht;
		QTextBlock::iterator it;
		for(it = block.begin(); !(it.atEnd()); ++it) {
			QTextFragment textFragment = it.fragment();
			if( textFragment.isValid() ) {
				//qDebug() << currentFragment.text();
				painter.drawText(x, y + ht, textFragment.text());
			}
		}
		//y += ht;
		block = block.next();
	}
#endif
#endif
}
void ViEditView::drawCursor()
{
	qDebug() << "m_cursorWidth = " << m_cursorWidth;
	QPainter painter(viewport());
	QRect r = cursorRect();
	r.setWidth(m_cursorWidth);
	switch( mode() ) {
	case CMD:
		r = QRect(r.left(), r.top() + r.height()/2, r.width(), r.height()/2);
		break;
	case INSERT:
		r.setWidth(1);
		break;
	}
	//qDebug() << painter.hasClipping();
	painter.fillRect(r, Qt::red);
#if 0
	painter.setPen(Qt::transparent);
	painter.setBrush(Qt::red);
	painter.drawRect(r);
#endif
}
